﻿using System;
using System.Collections.Generic;
using System.Xml;
using umbraco.BusinessLogic;

namespace uForum.Businesslogic {
    public class Topic {

        public int Id { get; private set; }
        public int ParentId { get; private set; }
        public string MemberId { get; private set; }

        public string Title { get; set; }
        public string UrlName { get; set; }
        public string Body { get; set; }
        public DateTime Created { get; private set; }
        public DateTime Updated { get; private set; }

        public string LatestReplyAuthor { get; set; }
        public int Replies { get; private set; }

        public bool Locked { get; set; }
        private Events _e = new Events();

        public bool Exists {
            get { return Id > 0; }
        }

        public void Delete() {
            DeleteEventArgs e = new DeleteEventArgs();
            FireBeforeDelete(e);
            if (!e.Cancel) {
                Data.SqlHelper.ExecuteNonQuery("DELETE FROM forumTopics WHERE id = " + Id);
                Id = 0;
            	var forum = new Forum(ParentId);
				if (forum.Exists) forum.Save();
            	FireAfterDelete(e);
            }
        }

        public void Lock() {
            LockEventArgs e = new LockEventArgs();
            FireBeforeLock(e);

            if (!e.Cancel) {
                Data.SqlHelper.ExecuteNonQuery("UPDATE forumTopics SET locked = 1 WHERE id = " + Id);
                Id = 0;
                FireAfterLock(e);
            }  
        }

        public void Save() {
            if (Id == 0) {

                if (!string.IsNullOrEmpty(Title) && !string.IsNullOrEmpty(Body)) {

                    
                    CreateEventArgs e = new CreateEventArgs();
                    FireBeforeCreate(e);
                    if (!e.Cancel) {

                        UrlName = umbraco.cms.helpers.url.FormatUrl(Title);

                        Data.SqlHelper.ExecuteNonQuery("INSERT INTO forumTopics (parentId, memberId, title, urlName, body, latestReplyAuthor, replies) VALUES(@parentId, @memberId, @title, @urlname, @body, @latestReplyAuthor, @replies)",
                            Data.SqlHelper.CreateParameter("@parentId", ParentId),
                            Data.SqlHelper.CreateParameter("@memberId", MemberId),
                            Data.SqlHelper.CreateParameter("@title", Title),
                            Data.SqlHelper.CreateParameter("@urlname", UrlName),
                            Data.SqlHelper.CreateParameter("@latestReplyAuthor", LatestReplyAuthor),    
                            Data.SqlHelper.CreateParameter("@body", Body),
                            Data.SqlHelper.CreateParameter("@replies", 0)
                            );

                        Created = DateTime.Now;
                        Updated = DateTime.Now;
                        Id = Data.SqlHelper.ExecuteScalar<int>("SELECT MAX(id) FROM forumTopics WHERE memberId = @memberId",
                            Data.SqlHelper.CreateParameter("@memberId", MemberId));

                        Forum f = new Forum(ParentId);

                        if (f.Exists) {
                            f.SetLatestTopic(Id);
                            f.SetLatestAuthor(MemberId);
                            f.LatestPostDate = DateTime.Now;
                            f.Save();
                        }

                        FireAfterCreate(e);
                    }
                }

            } else {

                UpdateEventArgs e = new UpdateEventArgs();
                FireBeforeUpdate(e);

                if (!e.Cancel) {

                    int totalComments = Data.SqlHelper.ExecuteScalar<int>("SELECT count(id) from forumComments where topicId = @id", Data.SqlHelper.CreateParameter("@id", Id));

                    UrlName = umbraco.cms.helpers.url.FormatUrl(Title);

                    Data.SqlHelper.ExecuteNonQuery("UPDATE forumTopics SET replies = @replies, parentId = @parentId, memberId = @memberId, title = @title, urlname = @urlname, body = @body, updated = @updated, locked = @locked, latestReplyAuthor = @latestReplyAuthor WHERE id = @id",
                        Data.SqlHelper.CreateParameter("@parentId", ParentId),
                        Data.SqlHelper.CreateParameter("@memberId", MemberId),
                        Data.SqlHelper.CreateParameter("@title", Title),
                        Data.SqlHelper.CreateParameter("@urlname", UrlName),
                        Data.SqlHelper.CreateParameter("@body", Body),
                        Data.SqlHelper.CreateParameter("@id", Id),
                        Data.SqlHelper.CreateParameter("@updated", DateTime.Now),
                        Data.SqlHelper.CreateParameter("@latestReplyAuthor", LatestReplyAuthor),
                        Data.SqlHelper.CreateParameter("@locked", Locked),
                        Data.SqlHelper.CreateParameter("@replies", totalComments)
                        );
                    FireAfterUpdate(e);
                }
            }
        }

        public List<Comment> Comments() {
            List<Comment> lc = new List<Comment>();

            umbraco.DataLayer.IRecordsReader dr = Data.SqlHelper.ExecuteReader(
                "SELECT * FROM forumComments WHERE topicId = " + Id
            );

            try {
                //Sql effiecient way of fetching collection of comments instead of one by one.. 
                while (dr.Read()) {
                    Comment c = new Comment();
                    c.Id = dr.GetInt("id");
                    c.TopicId = dr.GetInt("topicId");
                    c.MemberId = dr.GetString("memberId");

                    c.Body = dr.GetString("body");

                    c.Created = dr.GetDateTime("created");

                    lc.Add(c);
                }
            } catch (Exception ex) {
                Log.Add(LogTypes.Error, 0, string.Format("uForum: Failed to get comments for topic {0}. Exception Details: {1}",Id, ex));
            }
            return lc;
        }

        public XmlNode ToXml(XmlDocument d) {
            XmlNode tx = d.CreateElement("topic");
            
            tx.AppendChild(umbraco.xmlHelper.addTextNode(d, "title", Title));
            tx.AppendChild(umbraco.xmlHelper.addCDataNode(d, "body", Body));

            tx.AppendChild(umbraco.xmlHelper.addTextNode(d, "urlname", UrlName));
            tx.Attributes.Append(umbraco.xmlHelper.addAttribute(d, "id", Id.ToString()));
            tx.Attributes.Append(umbraco.xmlHelper.addAttribute(d, "parentId", ParentId.ToString()));
            tx.Attributes.Append(umbraco.xmlHelper.addAttribute(d, "memberId", MemberId));

            tx.Attributes.Append(umbraco.xmlHelper.addAttribute(d, "latestReplyAuthor", LatestReplyAuthor));
            
            tx.Attributes.Append(umbraco.xmlHelper.addAttribute(d, "created", Created.ToString()));
            tx.Attributes.Append(umbraco.xmlHelper.addAttribute(d, "updated", Updated.ToString()));

            tx.Attributes.Append(umbraco.xmlHelper.addAttribute(d, "locked", Locked.ToString()));
            tx.Attributes.Append(umbraco.xmlHelper.addAttribute(d, "replies", Replies.ToString()));
            return tx;
        }

        public static Topic Create(int forumId, string title, string body, string memberId) {
            Topic t = new Topic();
            t.ParentId = forumId;
            t.Title = title;
            t.Body = body;
            t.MemberId = memberId;
            t.LatestReplyAuthor = memberId;
            t.Replies = 0;
            t.Save();
            
            return t;
        }

        public Topic() { }
        
        public Topic(int topicId) {
            umbraco.DataLayer.IRecordsReader dr = Data.SqlHelper.ExecuteReader( "SELECT * FROM forumTopics WHERE id = " + topicId  );

            if (dr.Read()) {

                Id = dr.GetInt("id");
                ParentId = dr.GetInt("parentId");
                MemberId = dr.GetString("memberId");
                Replies = dr.GetInt("replies");
                Title = dr.GetString("title");
                Body = dr.GetString("body");
                LatestReplyAuthor = dr.GetString("latestReplyAuthor");
                Created = dr.GetDateTime("created");
                Updated = dr.GetDateTime("updated");

                UrlName = dr.GetString("urlName");

                Locked = dr.GetBoolean("locked");
            }

            dr.Close();
        }

        //Collections
        public static List<Topic> TopicsInForum(int forumId, int topicsPerPage, int page) {
            List<Topic> lt = new List<Topic>();
            umbraco.DataLayer.IRecordsReader dr = Data.SqlHelper.ExecuteReader(
                "SELECT TOP " + topicsPerPage + " id FROM forumTopics WHERE parentId = " + forumId + " ORDER BY updated DESC"
            );

            while (dr.Read() ) {
                lt.Add(new Topic( dr.GetInt("id")));
            }

            return lt;
        }

        //Collections
        public static List<Topic> TopicsInForum(int forumId) {
            List<Topic> lt = new List<Topic>();
            umbraco.DataLayer.IRecordsReader dr = Data.SqlHelper.ExecuteReader(
                "SELECT id FROM forumTopics WHERE parentId = " + forumId + " ORDER BY updated DESC"
            );

            while (dr.Read()) {
                lt.Add(new Topic(dr.GetInt("id")));
            }

            return lt;
        }

        //Collections
        public static List<Topic> Latest(int amount) {
            List<Topic> lt = new List<Topic>();
            umbraco.DataLayer.IRecordsReader dr = Data.SqlHelper.ExecuteReader(
                "SELECT TOP " + amount + " id FROM forumTopics ORDER BY updated DESC"
            );

            while (dr.Read()) {
                lt.Add(new Topic(dr.GetInt("id")));
            }

            return lt;
        }

        /* Events */
        public static event EventHandler<CreateEventArgs> BeforeCreate;
        protected virtual void FireBeforeCreate(CreateEventArgs e) {
            _e.FireCancelableEvent(BeforeCreate, this, e);
        }
        public static event EventHandler<CreateEventArgs> AfterCreate;
        protected virtual void FireAfterCreate(CreateEventArgs e) {
            if (AfterCreate != null)
                AfterCreate(this, e);
        }


        public static event EventHandler<DeleteEventArgs> BeforeDelete;
        protected virtual void FireBeforeDelete(DeleteEventArgs e) {
            _e.FireCancelableEvent(BeforeDelete, this, e);
        }
        public static event EventHandler<DeleteEventArgs> AfterDelete;
        protected virtual void FireAfterDelete(DeleteEventArgs e) {
            if (AfterDelete != null)
                AfterDelete(this, e);
        }


        public static event EventHandler<LockEventArgs> BeforeLock;
        protected virtual void FireBeforeLock(LockEventArgs e) {
            _e.FireCancelableEvent(BeforeLock, this, e);
        }
        public static event EventHandler<LockEventArgs> AfterLock;
        protected virtual void FireAfterLock(LockEventArgs e) {
            if (AfterLock != null)
                AfterLock(this, e);
        }


        public static event EventHandler<UpdateEventArgs> BeforeUpdate;
        protected virtual void FireBeforeUpdate(UpdateEventArgs e) {
            _e.FireCancelableEvent(BeforeUpdate, this, e);
        }
        public static event EventHandler<UpdateEventArgs> AfterUpdate;
        protected virtual void FireAfterUpdate(UpdateEventArgs e) {
            if (AfterUpdate != null)
                AfterUpdate(this, e);
        }
    }
}
